"""credits.py - a context for displaying a game's credits

This file is part of Gummworld2.

Gummworld2 is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Gummworld2 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with Gummworld2.  If not, see <http://www.gnu.org/licenses/>.
"""

import pygame

import gummworld2
from gummworld2 import State
from gummworld2 import data, model

import settings


KEYS_FASTER = (pygame.K_UP,pygame.K_RIGHT,pygame.K_w,pygame.K_d)
KEYS_SLOWER = (pygame.K_DOWN,pygame.K_LEFT,pygame.K_a,pygame.K_s)
KEYS_PAUSE = (pygame.K_SPACE,pygame.K_RETURN)

TEXT = """
John 11
When Jesus came, he found that Lazarus had lain in the grave four days already.
Jesus said, I am the resurrection, and the life: he that believeth in me, though he were dead, yet shall he live:
And whosoever liveth and believeth in me shall never die. Take ye away the stone.
Martha, the sister of him that was dead, saith unto him, Lord, by this time he stinketh: for he hath been dead four days.
Then they took away the stone from the place where the dead was laid. And Jesus lifted up his eyes, and said, Father, I thank thee that thou hast heard me.
And when he thus had spoken, he cried with a loud voice, Lazarus, come forth.
And he that was dead came forth.

John 14
Jesus saith, I am the way, the truth, and the life: no man cometh unto the Father, but by me.
Verily, verily, I say unto you, He that believeth on me, the works that I do shall he do also; and greater works than these shall he do; because I go unto my Father.

Acts 14
When they were escaped, then they knew that the island was called Melita.
And the barbarous people shewed us no little kindness: for they kindled a fire, and received us every one, because of the present rain, and because of the cold.
And when Paul had gathered a bundle of sticks, and laid them on the fire, there came a viper out of the heat, and fastened on his hand.
And when the barbarians saw the venomous beast hang on his hand, they said among themselves, No doubt this man is a murderer, whom, though he hath escaped the sea, yet vengeance suffereth not to live.
And he shook off the beast into the fire, and felt no harm.
Howbeit they looked when he should have swollen, or fallen down dead suddenly: but after they had looked a great while, and saw no harm come to him, they changed their minds, and said that he was a god.

Acts 5
Believers were the more added to the Lord, multitudes both of men and women.
Insomuch that they brought forth the sick into the streets, and laid them on beds and couches, that at the least the shadow of Peter passing by might overshadow some of them.
There came also a multitude out of the cities round about unto Jerusalem, bringing sick folks, and them which were vexed with unclean spirits: and they were healed every one.
Then the high priest rose up, and all they that were with him, (which is the sect of the Sadducees,) and were filled with indignation,
And laid their hands on the apostles, and put them in the common prison.
But the angel of the Lord by night opened the prison doors, and brought them forth, and said,
Go, stand and speak in the temple to the people all the words of this life.
But when the officers came, and found them not in the prison, they returned, and told,
Saying, The prison truly found we shut with all safety, and the keepers standing without before the doors: but when we had opened, we found no man within.
Then went the captain with the officers, and brought them without violence: for they feared the people, lest they should have been stoned.
And when they had brought them, they set them before the council: and the high priest asked them,
Saying, Did not we straitly command you that ye should not teach in this name? and, behold, ye have filled Jerusalem with your doctrine, and intend to bring this man's blood upon us.
Then Peter and the other apostles answered and said, We ought to obey God rather than men.

John 3
For God so loved the world, that he gave his only begotten Son, that whosoever believeth in him should not perish, but have everlasting life.
For God sent not his Son into the world to condemn the world; but that the world through him might be saved.

John 1
But as many as received him, to them gave he power to become the sons of God, even to them that believe on his name:
Which were born, not of blood, nor of the will of the flesh, nor of the will of man, but of God.
And the Word was made flesh, and dwelt among us, (and we beheld his glory, the glory as of the only begotten of the Father,) full of grace and truth.

Acts 2
But this is that which was spoken by the prophet Joel;
And it shall come to pass in the last days, saith God, I will pour out of my Spirit upon all flesh: and your sons and your daughters shall prophesy, and your young men shall see visions, and your old men shall dream dreams:
And on my servants and on my handmaidens I will pour out in those days of my Spirit; and they shall prophesy:
And I will shew wonders in heaven above, and signs in the earth beneath; blood, and fire, and vapour of smoke:
The sun shall be turned into darkness, and the moon into blood, before that great and notable day of the Lord come:
And it shall come to pass, that whosoever shall call on the name of the Lord shall be saved.

Psalms 100
A Psalm of praise. Make a joyful noise unto the LORD, all ye lands.
Serve the LORD with gladness: come before his presence with singing.
Know ye that the LORD he is God: it is he that hath made us, and not we ourselves; we are his people, and the sheep of his pasture.
Enter into his gates with thanksgiving, and into his courts with praise: be thankful unto him, and bless his name.
For the LORD is good; his mercy is everlasting; and his truth endureth to all generations.
""".split('\n')

while TEXT[0] == '':
    del TEXT[0]
while TEXT[-1] == '':
    del TEXT[-1]


class Credits(gummworld2.Engine):
    
    def __init__(self):
        w,h = settings.resolution
        if w >= 800:
            w = 500
        
        self.ui_font_height = 13
        self.font_height = 18
        self.ui_font = pygame.font.Font(
            data.filepath('font', 'Vera.ttf'), self.ui_font_height)
        self.font = pygame.font.Font(
            data.filepath('font', 'Vera.ttf'), self.font_height)
        self.font_size = gummworld2.Vec2d(self.font.size(' '))
        self.word_spacing = self.font_size.x
        self.line_spacing = 0
        self.para_spacing = int(round(self.font_size.y / 2.0))
        
        screen_rect = pygame.Rect((0,0), settings.resolution)
        view_rect = pygame.Rect(0,0, w,h)
        view_rect.center = screen_rect.center
        camera_target = model.Object(view_rect.center)
        
        gummworld2.Engine.__init__(self,
            resolution=settings.resolution,
            camera_view_rect=view_rect, camera_target=camera_target,
            update_speed=60, frame_speed=120,
        )
        
        self.fgcolor = pygame.Color('orange')
        self.frame_color = pygame.Color('darkgrey')
        self.tiles = []
        self.visible = []
        self.text_stream = self.gen_text()
        self.next_tile = self.text_stream.next()
        
        self.paused = False
        self.mode = 0
        self.rest = 1.0
        self.speed = 0.0
        self.speed0 = 0.1   # cruising speed, game loop ramps up to this
        self.ui = {
            'speed' : None,
            'tip0' : None,
            'tip1' : None,
            'tip2' : None,
        }
    
    def gen_text(self):
        # Make the word objects to put in BasicMap
        fgcolor = self.fgcolor
        view_rect = self.camera.rect
        x = 0
        y = view_rect.y + 5*(self.font_size.y + self.line_spacing)
        has_text = False
        for line in TEXT:
            words = line.split(' ')
            tile = model.Object((0,0))
            tile.image = pygame.Surface((view_rect.w,self.font_size.y+self.line_spacing))
            tile.image.set_alpha(0)
            tile.rect = tile.image.get_rect(topleft=(0,y))
            tile.position = tile.rect.center
            has_text = False
            for word in words:
                has_text = True
                # blank line
                if word == '':
                    continue
                # apply formatting
                if word[0] == '#':
                    pass
                # make image
                im = self.font.render(word, 1, fgcolor)
                r = im.get_rect(topleft=(x,y))
                # word wrap
                if x + r.w > tile.rect.right:
                    yield tile
                    x = 0
                    y += self.line_spacing + r.h
                    tile = model.Object((0,0))
                    tile.image = pygame.Surface((view_rect.w,self.font_size.y+self.line_spacing))
                    tile.image.set_alpha(0)
                    tile.rect = tile.image.get_rect(topleft=(0,y))
                    tile.position = tile.rect.center
                tile.image.blit(im, (x,0))
                # advance word spacing
                x += r.w + self.word_spacing
            if has_text:
                yield tile
            # advance new line
            x = 0
            y += self.para_spacing + r.h
    
    def auto_exit(self, dt):
        self.pop()
    
    def update(self, dt):
        self.update_visible()
        self.update_mode(dt)
        self.camera.update()
    
    def update_visible(self):
        if self.mode < 1:
            return
        
        # Add the next tile when it becomes visible. If next_tile is None
        # then there are no more to get.
        next_tile = self.next_tile
        if next_tile and next_tile.rect.colliderect(self.camera.rect):
            self.visible.append(next_tile)
            try:
                self.next_tile = self.text_stream.next()
            except StopIteration:
                self.next_tile = None
        # Remove any tiles that are no longer visible.
        for tile in self.visible[:]:
            if not tile.rect.colliderect(self.camera.rect):
                self.visible.remove(tile)
        
        # If all visible tiles have been removed we are done.
        if not self.visible:
            self.stop()

    def cascade_alpha(self, quantity=1):
        fade_out_start = 2 * (self.font_size.y + self.line_spacing)
        fade_in_start = 60
        cy = self.camera.rect.y
        prev_alpha = fade_in_start
        num_changed = 0
        for tile in self.visible:
            alpha = tile.image.get_alpha()
            if alpha is None:
                alpha = 255
            if alpha < 255:
                num_changed += 1
            if tile.rect.y - cy < fade_out_start:
                continue
            if prev_alpha >= fade_in_start:
                prev_alpha = alpha
                if alpha != None:
                    alpha += quantity
                    if alpha >= 255:
                        alpha = 255
                tile.image.set_alpha(alpha)
        return num_changed
    
    def inc_alpha(self):
        cam_rect = self.camera.rect
        line_height = self.font_size.y + self.line_spacing
        fade_in_end = cam_rect.height - 4 * line_height
        screen_height = cam_rect.height
        fade_in_distance = screen_height - fade_in_end
        cy = cam_rect.y
        for tile in self.visible:
            tile_pos = tile.rect.y - cy
            if tile_pos >= fade_in_end and tile.image.get_alpha() < 255:
                alpha = int(255 * float(screen_height-tile_pos) / fade_in_distance)
                if alpha > 255:
                    alpha = 255
                elif alpha < 0:
                    alpha = 0
                tile.image.set_alpha(alpha)
    
    def dec_alpha(self):
        fade_out_start = 3 * (self.font_size.y + self.line_spacing)
        cy = self.camera.rect.y
        for tile in self.visible:
            tile_pos = tile.rect.y - cy
            if tile_pos <= fade_out_start:
                alpha = int(255 * (float(tile_pos) / float(fade_out_start)))
                if alpha < 0:
                    alpha = 0
                elif alpha > 255:
                    alpha = 255
                tile.image.set_alpha(alpha)
    
    def render_speed(self):
        speed = 'Speed {0:0.2f}'.format(self.speed)
        self.ui['speed'] = self.ui_font.render(speed, 1, self.fgcolor)
    
    def render_tip(self, dt):
        self.ui['tip0'] = self.ui_font.render('Faster: Up', 1, self.fgcolor)
        self.ui['tip1'] = self.ui_font.render('Slower: Down', 1, self.fgcolor)
        self.ui['tip2'] = self.ui_font.render('Pause: Space', 1, self.fgcolor)
        self.clock.unschedule(self.render_tip)
    
    def update_mode(self, dt):
        if self.mode == 0:
            # Rest a moment for display to settle
            self.rest -= dt
            if self.rest <= 0.0:
                self.mode += 1
        elif self.mode == 1:
            # Swipe in first page
            num_changed = self.cascade_alpha(64) #255)
            if num_changed == 0:
                self.mode += 1
        elif self.mode == 2:
            # Ramp speed up to speed0
            self.speed += dt * 0.025
            self.render_speed()
            if self.speed >= self.speed0:
                self.speed = self.speed0
                self.mode += 1
                self.clock.schedule_interval(self.render_tip, 7.0)
            self.camera.position += (0,self.speed)
            self.inc_alpha()
            self.dec_alpha()
        elif self.mode == 3:
            if self.paused:
                return
            # Scroll through the text
            self.camera.position += (0,self.speed)
            self.inc_alpha()
            self.dec_alpha()
    
    def draw(self, dt):
        screen = self.screen
        view = self.camera.view
        screen.clear()
        cy = self.camera.rect.y
        for tile in self.visible:
            view.blit(tile.image, tile.rect.move(0,-cy))
        # Draw UI
        y = 0
        for name in ('speed','tip0','tip1','tip2'):
            im = self.ui[name]
            if im is None:
                continue
            screen.blit(im, (0,y))
            y += im.get_height()
        screen.flip()
    
    def on_key_down(self, unicode, key, mod):
        if key in KEYS_FASTER:
            self.speed += self.speed * 1.0/4.0
            self.render_speed()
        elif key in KEYS_SLOWER:
            if self.speed >= 0.05:
                self.speed -= self.speed * 1.0/5.0
                self.render_speed()
        elif key in KEYS_PAUSE:
            self.paused = not self.paused

    def stop(self):
        self.pop()
