"""lighting.py - Variable lighting

This is an idea I had for variable lighting. Can't figure out an efficient way
to integrate it. Calculating the many nodes on screen is very expensive.
"""

import math

from gummworld2 import geometry

import terrainnodes


class PointSource(geometry.CircleGeometry):
    
    def __init__(self, pos, magnitude=1.0, scale=32):
        super(PointSource, self).__init__(pos, 1)
        self.max_radius = 20
        self.magnitude = magnitude
        self.scale = scale
        
    def _get_magnitude(self):
        return self._magnitude
    def _set_magnitude(self, val):
        if val > 1.0:
            val = 1.0
        elif val < 0.0:
            val = 0.0
        self._magnitude = val
        self.radius = self.max_radius * terrainnodes.NODE_SIZE()[0] * val
    magnitude = property(_get_magnitude, _set_magnitude, "magnitude is 0.0 to 1.0")
    
    def _get_scale(self):
        return self._scale
    def _set_scale(self, val):
        self._scale = float(val)
    scale = property(_get_scale, _set_scale)
    
    def intensity_at(self, pos):
        dist = geometry.distance(self.origin, pos) / float(self._scale)
        dist_squared = dist * dist
        return self.magnitude / dist_squared * 4 * math.pi


if __name__ == '__main__':
    import pygame
    pygame.init()
    screen = pygame.display.set_mode((800,600))
    screen_rect = screen.get_rect()
    clock = pygame.time.Clock()
    rect = pygame.Rect(0,0,20,20)
    lamp = PointSource(screen_rect.center, scale=32)
    lamp_rect = pygame.Rect(lamp.rect.center, (10,10))
    black = pygame.Color('black')
    red = pygame.Color('red')
    white = pygame.Color('white')
    while True:
        clock.tick(30)
        pygame.event.clear()
        rect.bottomright = pygame.mouse.get_pos()
        screen.fill(black)
        screen.fill(white, lamp_rect)
        if screen_rect.contains(rect):
            screen.fill(red, rect)
            intensity = lamp.intensity_at(rect.center)
            n = int(round(255 - 255 * intensity))
            if n > 255:
                n = 255
            if n < 0:
                n = 0
            c = (n,n,n)
            screen.fill(c, rect, pygame.BLEND_RGBA_SUB)
        pygame.display.flip()
